import time
import unittest
from typing import Optional

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from selenium import webdriver
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

from ui_tests.main import get_app
from ui_tests_shared.declarations import Declarations


class TestCaseWithApp(unittest.TestCase):
    options = webdriver.ChromeOptions()
    options.add_experimental_option('useAutomationExtension', False)
    options.add_experimental_option("excludeSwitches", ["enable-automation"])

    _config: Optional[ConfigurationABC] = None
    _services: Optional[ServiceProviderABC] = None
    _driver = webdriver.Chrome(options=options)

    _is_logged_in = False

    @classmethod
    def setUpClass(cls):
        if Declarations.app is None:
            get_app()

        cls._config = Declarations.app.config
        cls._services = Declarations.app.services
        cls._login()

    @classmethod
    def _login(cls):
        if cls._is_logged_in:
            return

        # use full xpath: https://stackoverflow.com/questions/71179006/how-can-selenium-python-chrome-find-web-elements-visible-in-dev-tools-but-no

        cls._driver.get('https://discord.com/login')

        WebDriverWait(cls._driver, 20).until(expected_conditions.presence_of_element_located((By.NAME, 'email')))

        mail = cls._driver.find_element(By.NAME, 'email')
        mail.clear()
        mail.send_keys("dev.sven.heidemann@sh-edraft.de")
        mail.send_keys(Keys.RETURN)

        pw = cls._driver.find_element(By.NAME, 'password')
        pw.clear()
        pw.send_keys("Heidemann1410")
        pw.send_keys(Keys.RETURN)
        time.sleep(1)
        pw.send_keys(Keys.RETURN)

        WebDriverWait(cls._driver, 20).until(expected_conditions.url_matches('https://discord.com/channels/@me'))
        time.sleep(4)
